/* 
 * dup2.c
 *
 * x-kernel v3.2
 *
 * Copyright (c) 1993,1991,1990  Arizona Board of Regents
 *
 *
 * $Revision: 1.2 $
 * $Date: 1996/05/26 22:37:23 $
 */


#include "xkernel.h"
#include "dup2.h"
#include "dup2_i.h"

#ifdef __STDC__

static void		protlFuncInit( XObj );
static void		sessnInit( XObj );
static XObj		dup2CreateSessn( XObj, XObj, XObj, XObj );
static XObj		dup2Open( XObj, XObj, XObj, Part * );
static xkern_return_t	dup2OpenDisable( XObj, XObj, XObj, Part * );
static xkern_return_t	dup2OpenDone( XObj, XObj, XObj, XObj );
static xkern_return_t	dup2OpenEnable( XObj, XObj, XObj, Part * );
static int		dup2ControlProtl( XObj, int, char *, int );
static int		dup2ControlSessn( XObj, int, char *, int );
static xkern_return_t	dup2Close( XObj );
static xkern_return_t	dup2Pop( XObj, XObj, Msg *, void * );
static xmsg_handle_t	dup2Push( XObj, Msg * );
static xkern_return_t	dup2ProtlDemux( XObj, XObj, Msg * );
static xkern_return_t	dup2SessnDemux( XObj, XObj, Msg * );

#else

static void		protlFuncInit();
static void		sessnInit();
static XObj		dup2CreateSessn();
static XObj		dup2Open();
static xkern_return_t	dup2OpenDisable();
static xkern_return_t	dup2OpenDone();
static xkern_return_t	dup2OpenEnable();

#endif /* __STDC__ */


int tracedup2p;

void
dup2_init( self )
    XObj 	self;
{
    PState	*ps;
    
    xTraceP0(self, TR_GROSS_EVENTS, "init");
    xAssert(xIsProtocol(self));
    if ( ! xIsProtocol(xGetDown(self, 0)) ) {
	xError("DUP2 down vector is misconfigured");
	return;
    }
    ps = X_NEW(PState);
    self->state = (VOID *)ps;
    protlFuncInit(self);
    ps->activeMap = mapCreate(DUP2_ACT_MAP_SZ, sizeof(XObj));
    ps->passiveMap = mapCreate(DUP2_PAS_MAP_SZ, sizeof(XObj));
    ps->actExistMap = mapCreate(DUP2_ACT_EXIST_MAP_SZ, ACT_PART_SZ);
    ps->pasExistMap = mapCreate(DUP2_PAS_EXIST_MAP_SZ, PAS_PART_SZ);
}


static XObj
dup2Open( self, hlpRcv, hlpType, p )
    XObj	self, hlpRcv, hlpType;
    Part 	*p;
{
    XObj	s, lls;
    PState	*ps = (PState *)self->state;
    Part	part[2];
    char	key[ACT_PART_SZ];
    char	*posn = key;
    Bind	binding;

    xTraceP0(self, TR_MAJOR_EVENTS, "open");

    bcopy((char *)p, (char *)part, sizeof(part));
    bzero(key, ACT_PART_SZ);
    bcopy(partPop(part[1]), posn, sizeof(long));
    posn += sizeof(long);
    bcopy(partPop(part[1]), posn, sizeof(IPhost));
    posn += sizeof(IPhost);
    bcopy(partPop(part[0]), posn, sizeof(long));
    posn += sizeof(long);
    bcopy(partPop(part[0]), posn, sizeof(IPhost));
    posn += sizeof(IPhost);

    if ( mapResolve(ps->actExistMap, &key, &s) == XK_SUCCESS ) {
	xTraceP0(self, TR_MORE_EVENTS, "open -- found an existing one");
    } else {
	if ((lls = xOpen(self, hlpType, xGetDown(self, 0), p)) == ERR_XOBJ ){
            xTraceP0(self, TR_ERRORS, "open: could not open lower session");
            return ERR_XOBJ;
	}
    	if ( mapResolve(ps->activeMap, &lls, &s) == XK_SUCCESS ) {
            xTraceP0(self, TR_MORE_EVENTS, "open -- found an existing one");
    	} else {
            xTraceP0(self, TR_MAJOR_EVENTS, "open -- creating a new one");
            if ((s = dup2CreateSessn(self, hlpRcv, hlpType, lls)) == ERR_XOBJ) {
            	xClose(lls);
            }
	    binding = mapBind(ps->actExistMap, key, s);
	    if ( binding == ERR_BIND ) {
	    	xTraceP0(self, TR_ERRORS, "mapBind fails");
	    	return ERR_XOBJ;
	    }
        }
    }
    return s;
}


static XObj
dup2CreateSessn( self, hlpRcv, hlpType, lls )
    XObj 	self, hlpRcv, hlpType, lls;
{
    XObj	s;
    PState	*ps = (PState *)self->state;
    SState	*ss;
    
    if ( (s = xCreateSessn(sessnInit, hlpRcv, hlpType, self, 1, &lls))
			== ERR_XOBJ ) {
	xTraceP0(self, TR_ERRORS, "xCreateSessn fails");
	return ERR_XOBJ;
    }
    s->binding = mapBind(ps->activeMap, &lls, s);
    if ( s->binding == ERR_BIND ) {
	xTraceP0(self, TR_ERRORS, "mapBind fails in createSessn");
	return ERR_XOBJ;
    }
    /*
     * The lower sessions' up field is made to point to this
     * dup2 session (not the protocol)
     */
    xSetUp(lls, s);
    ss = X_NEW(SState);
    s->state = (VOID *)ss;
    xTraceP1(self, TR_MAJOR_EVENTS, "open returns %x", s);
    return s;
}


static int
dup2ControlSessn( s, opcode, buf, len )
    XObj 	s;
    int 	opcode, len;
    char 	*buf;
{
    xTraceP0(s, TR_EVENTS, "controlsessn");
    switch ( opcode ) {
      default:
	/*
	 * All other opcodes are forwarded to the lower session.  
	 */
	return xControl(xGetDown(s, 0), opcode, buf, len);
    }
}


static int
dup2ControlProtl( self, opcode, buf, len )
    XObj 	self;
    int 	opcode, len;
    char 	*buf;
{
    /*
     * All opcodes are forwarded to the lower protocol
     */
    return xControl(xGetDown(self, 0), opcode, buf, len);
}


static xkern_return_t
dup2OpenEnable( self, hlpRcv, hlpType, p )
    XObj 	self, hlpRcv, hlpType;
    Part 	*p;
{
    PState	*ps = (PState *)self->state;
    Part	part[2];
    char	key[PAS_PART_SZ];
    xkern_return_t ret;
    Bind	binding;
    
    bcopy((char *)p, (char *)part, sizeof(part));
    bzero(key, PAS_PART_SZ);
    bcopy(partPop(part[0]), key, sizeof(long));

    xTraceP0(self, TR_MAJOR_EVENTS, "open enable");
    if ( mapResolve(ps->pasExistMap, &key, NULL) == XK_SUCCESS ) {
	return XK_SUCCESS;
    } else {
    	ret =  defaultVirtualOpenEnable(self, ps->passiveMap, hlpRcv, hlpType,
				    self->down, p);
	if (ret == XK_SUCCESS) {
            binding =  mapBind(ps->pasExistMap, key, NULL);
            if ( binding == ERR_BIND ) {
                xTraceP0(self, TR_ERRORS,
                    "openenable -- binding failed!");
                return XK_FAILURE;
            }
	}
	return ret;
    }
}


static xkern_return_t
dup2OpenDisable( self, hlpRcv, hlpType, p )
    XObj 	self, hlpRcv, hlpType;
    Part 	*p;
{
    PState	*ps = (PState *)self->state;

    return defaultVirtualOpenDisable(self, ps->passiveMap, hlpRcv, hlpType,
				     self->down, p);
}


static xkern_return_t
dup2Close( s )
    XObj 	s;
{
    PState		*ps;
    xkern_return_t	res;
    XObj		lls;
    xkern_return_t      ret;
    char        	buffer[100];
    Part        	part[2];
    char        	key[ACT_PART_SZ];
    char        	*posn = key;

    
    xTraceP1(s, TR_MAJOR_EVENTS, "close of session %lx", (u_long)s);
    xAssert(s->rcnt == 0);
    ps = (PState *)xMyProtl(s)->state;
    res = mapRemoveBinding(ps->activeMap, s->binding);
    xAssert( res != XK_FAILURE );
    lls = xGetDown(s, 0);
    xAssert(xIsSession(lls));

    xControl(lls,GETPARTICIPANTS,buffer,sizeof(buffer));
    partInternalize(part,buffer);

    bzero(key, ACT_PART_SZ);
    bcopy(partPop(part[1]), posn, sizeof(long));
    posn += sizeof(long);
    bcopy(partPop(part[1]), posn, sizeof(IPhost));
    posn += sizeof(IPhost);
    bcopy(partPop(part[0]), posn, sizeof(long));
    posn += sizeof(long);
    bcopy(partPop(part[0]), posn, sizeof(IPhost));
    posn += sizeof(IPhost);

    ret = mapUnbind(ps->actExistMap, key);
    if ( ret == XK_FAILURE ) {
        xTraceP0(s, TR_ERRORS, "mapUnbind fails");
        return XK_FAILURE;
    }

    xSetUp(lls, xMyProtl(s));
    xClose(lls);
    xDestroy(s);
    return XK_SUCCESS;
}


static xmsg_handle_t
dup2Push( s, msg )
    XObj	s;
    Msg 	*msg;
{
    xTraceP0(s, TR_EVENTS, "push");
    return xPush(xGetDown(s, 0), msg);
}


static xkern_return_t
dup2OpenDone( self, lls, llp, hlpType )
    XObj	self, lls, llp, hlpType;
{
    XObj	s;
    PState	*ps = (PState *)self->state;
    Enable	*e;
    Bind	binding;
    char	buffer[100];
    Part        part[2];
    char        key[ACT_PART_SZ];
    char        *posn = key;

    
    xTraceP0(self, TR_MAJOR_EVENTS, "In openDone");
    if ( self == hlpType ) {
	xTraceP0(self, TR_ERRORS, "self == hlpType in openDone");
	return XK_FAILURE;
    }
    /*
     * check for openEnable
     */
    
    if ( mapResolve(ps->passiveMap, &hlpType, &e) == XK_FAILURE ) {
	/* 
	 * This shouldn't happen
	 */
	xTraceP0(self, TR_ERRORS,
		"openDone: Couldn't find hlp for incoming session");
	return XK_FAILURE;
    }
    if ( (s = dup2CreateSessn(self, e->hlpRcv, e->hlpType, lls))
							== ERR_XOBJ ) {
	return XK_FAILURE;
    }
    
    xControl(lls,GETPARTICIPANTS,buffer,sizeof(buffer));
    partInternalize(part,buffer);

    bzero(key, ACT_PART_SZ);
    bcopy(partPop(part[1]), posn, sizeof(long));
    posn += sizeof(long);
    bcopy(partPop(part[1]), posn, sizeof(IPhost));
    posn += sizeof(IPhost);
    bcopy(partPop(part[0]), posn, sizeof(long));
    posn += sizeof(long);
    bcopy(partPop(part[0]), posn, sizeof(IPhost));
    posn += sizeof(IPhost);

    binding = mapBind(ps->actExistMap, key, s);
    if ( binding == ERR_BIND ) {
        xTraceP0(self, TR_ERRORS, "mapBind fails");
        return XK_FAILURE;
    }

    xDuplicate(lls);
    xTraceP0(self, TR_EVENTS,
	    "passively opened session successfully created");
    return xOpenDone(e->hlpRcv, s, self);
}


static xkern_return_t
dup2ProtlDemux( self, lls, m )
    XObj	self, lls;
    Msg		*m;
{
    xTraceP0(self, TR_ERRORS, "protlDemux called!!");
    return XK_SUCCESS;
}


/* 
 * dup2Pop and dup2SessnDemux must be used (i.e., they can't be
 * bypassed) for the UPI reference count mechanism to work properly. 
 */
static xkern_return_t
dup2Pop( s, lls, msg, h )
    XObj 	s, lls;
    Msg 	*msg;
    VOID	*h;
{
    xTraceP0(s, TR_EVENTS, "pop");
    return xDemux(s, msg);
}


static xkern_return_t
dup2SessnDemux( self, lls, msg )
    XObj 	self, lls;
    Msg 	*msg;
{
    xTraceP0(self, TR_EVENTS, "session demux");
    return xPop(self, lls, msg, 0);
}


static void
sessnInit( s )
    XObj 	s;
{
    xAssert(xIsSession(s));
    
    s->push = dup2Push;
    s->pop = dup2Pop;
    s->close = dup2Close;
    s->control = dup2ControlSessn;
    /* 
     * DUP2 sessions will look like a protocol to lower sessions, so we
     * need a demux function
     */
    s->demux = dup2SessnDemux;
}


static void
protlFuncInit(p)
    XObj p;
{
    xAssert(xIsProtocol(p));

    p->control = dup2ControlProtl;
    p->open = dup2Open;
    p->openenable = dup2OpenEnable;
    p->opendisable = dup2OpenDisable;
    p->demux = dup2ProtlDemux;
    p->opendone = dup2OpenDone;
}
