/*     
 * $RCSfile: rip_i.h,v $
 *
 * x-kernel v3.2
 *
 * Copyright (c) 1993,1991,1990  Arizona Board of Regents
 *
 *
 * $Log: rip_i.h,v $
 * Revision 1.2  1994/09/16 16:27:45  menze
 * 'SEC' renamed 'ONESEC' to avoid name conflicts
 *
 */


#ifndef rip_i_h
#define rip_i_h


#include "ip.h"
#include "riproute.h"
#include "riproute_i.h"

#define UP_RIP          520     /* route information exchange (RIP)    */
#define OK		1
#define SYSERR		0

#define IP_ALEN		4
#define IP_CLASSA(x)    (((x).a & 0x80) == 0x00) /* IP Class A address   */
#define IP_CLASSB(x)    (((x).a & 0xc0) == 0x80) /* IP Class B address   */
#define IP_CLASSC(x)    (((x).a & 0xe0) == 0xc0) /* IP Class C address   */
#define IP_CLASSD(x)    (((x).a & 0xf0) == 0xe0) /* IP Class D address   */
#define IP_CLASSE(x)    (((x).a & 0xf8) == 0xf0) /* IP Class E address   */


#define	RIPHSIZE	4	/* size of the header in octets		*/
#define	RIP_VERSION	1	/* RIP version number			*/
#define	AF_INET		2	/* Address Family for IP		*/

/* RIP commands codes: */

#define	RIP_REQUEST	1	/* message requests routes		*/
#define	RIP_RESPONSE	2	/* message contains route advertisement	*/

/* one RIP route entry */


#define RIPNULLADDR 8

struct	riprt {
	short	rr_family;	/* 4BSD Address Family			*/
	short	rr_mbz;		/* must be zero				*/
	IPhost	rr_addr;	/* ip address				*/
	char	rr_mbz2[RIPNULLADDR];	/* must be zero				*/
	int	rr_metric;	/* distance (hop count) metric		*/
};

#define	MAXRIPROUTES	25	/* MAX routes per packet		*/
#define	RIP_INFINITY	16	/* dest is unreachable			*/

#define TENTHSEC	100*1000 	/* a tenth of a second in usec */
#define ONESEC		1000*1000 	/* a second in usec */
#define RIPINT		30 	/* interval to send (secs)	*/
#define	RIPDELTA	 5	/* stray RIPINT +/- this (secs)*/
#define	RIPOUTMIN	 5     	/* min interval between updates (secs)*/

#define RIPAGEINT	60 	/* interval to send ( secs)	*/
#define RIPPRINTINT	30 	/* interval to send ( secs)	*/

#define RIPRTTL		3	/* route ttl (mins)			*/
#define	RIPZTIME	2	/* zombie route lifetime (mins)		*/


/* RIP packet structure */
struct	rip {
	char	rip_cmd;	/* RIP command				*/
	char	rip_vers;	/* RIP_VERSION, above			*/
	short	rip_mbz;	/* must be zero				*/
	struct	riprt	rip_rts[MAXRIPROUTES];
};

/* RIP process definitions */

#define	MAXNRIP		5		/* max # of packets per update	*/

#define	RIPMAXINTER	10		/* max # of interfaces	*/

typedef struct ifc {
    IPhost      net;
    IPhost      host;
    IPhost      mask;
    IPhost      bcast;	 /* broadcast address for this net */
    XObj	bcast_s; /* udp bcast session for this net */
    int         subnet;  /* does this interface use subnetting? */
    VOID 	*vnet_id; /* magic cookie from vnet */
    int		active;
} Interface;

/* Per-interface RIP send data */
struct	rq {
	int		rq_active;	/* TRUE => this data is active	*/
	XObj		rq_udp_s;		/* udp session */
/* what we've built already:		*/
	struct rip	rq_pep[MAXNRIP]; /* packets to send	*/
        int             rq_len[MAXNRIP]; /* length of each       */
	int		rq_cur;		/* current working packet	*/
/* current packet */
	int		rq_nrts;	/* # routes this packet		*/
	struct	rip	*rq_prip;	/* current rip data		*/
};


typedef struct pstate {
	int		active;		/* is intialization done? */ 
	int		numNif;		/* number of protocols below vnet */
        Interface	nif[RIPMAXINTER];	/* duplicate of vnet info */
	RipRouteTable	rtTbl;		/* rips copy of the routing table */
	Semaphore	riplock;	/* ripin/ripout synchronization	*/
	int		gateway;	/* 	*/
	int		dorip;		/* TRUE => do active RIP	*/
	XObj		self;
	int		printTime;
	int		printInterval;
	int		ageTime;
	int		ageInterval;
	int		outputTime;
	int		outputInterval;
	int		outputForced;
} PSTATE; 

#endif /* ! rip_i_h */

