#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <pwd.h>
#include <fcntl.h>
#include "xkernel.h"
#include "vtap.h"
#include "rl_i.h"
#include "crypt.h"
#include "tcp.h"
#include "ipsec.h"



/* log:
 * 	used popen()
 *	used pipe()
 *	used socketpair() to get better buffering
 */

#define DEFAULT_SERV_ADDR "192.12.69.50"
#define DEFAULT_SHELL_COMMAND "./newlogin/login"

#define LOGIN_PORT IPPORT_LOGINSERVER


int s_pipe(int fd[2]);

#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

#ifdef SUNOS
#include <unistd.h>
#include <memory.h>
#include <ctype.h>
int setpgrp P_((int, int));
void perror P_((char *));
int select P_((int, fd_set*, fd_set*, fd_set*, const struct timeval*));
int getdtablesize (void);
int socketpair P_((int, int, int, int *));
int setsockopt P_((int, int, int, char *, int));
#endif


#ifdef SOLARIS
#include <unistd.h>
#include <ctype.h>
#endif

/* rlogin/rl.c */
void rl_init P_((XObj self));
static long mycopypop P_((void *dest, char *src, long count, void *arg));
static void mycopypush P_((void *src, char *dst, long count, void *arg));
void rlogin_msgtest P_((void));
void server_init P_((Event e, void *tself));
static void sessnInit P_((XObj s));
static void protInit P_((XObj p));
static XObj rloginOpen P_((XObj self, XObj hlpRcv, XObj hlpType, Part *p));
static XObj rloginCreateSession P_((XObj self, XObj hlpRcv, XObj hlpType, XObj lls));
xkern_return_t rlogin_opendone P_((XObj self, XObj lls, XObj llp, XObj hlpType));
void server_shell_init P_((XObj s, char * ));
int hostname_islegal P_((char *hostname));
void shell_poll P_((Event e, void *vself));
static xkern_return_t rloginDemux P_((XObj self, XObj lls, Msg *msg));
int string_split_space P_((char *string, char *s, char *t));


extern int tracerlp; int tracerlp;

int isserver = 1;
int noopenmode = 0;
int noipsecmode = 0;
int vtapmode = 0;
char *specified_shell = NULL;

Msg rlogin_msg;
static int debuglevel = 0;

#define RDEBUG(x) if (debuglevel >= (x))

static long 
mycopypop(dest, src, count, arg)
     void *dest;
     char *src;
     long count;
     void *arg;
{
    bcopy(src, (char*)dest, count);
    return count;
}

static void 
mycopypush(src, dst, count, arg)
     void *src;
     char *dst;
     long  count;
     void *arg;
{
    bcopy((char *)src, dst, count);
}



void 
rl_init(XObj self)
{
    PState *ps;
    char *pc;
    int i;
    XObj rlogin_llp;

    xTrace0(rlp, TR_GROSS_EVENTS, "RLOGIN init");
    xAssert(xIsProtocol(self));

    if (!xIsProtocol(rlogin_llp = xGetDown(self, 0))) {
	Kabort("RLOGIN could not get lower protocol");
    }

    /* set up the maps */
    ps = X_NEW(PState);
    self->state = (VOID *)ps;
    ps->activeMap = mapCreate(RLOGIN_ACT_MAP_SZ, sizeof(XObj));
    ps->passiveMap = mapCreate(RLOGIN_PAS_MAP_SZ, sizeof(XObj));

    /* parse arguments */
    for (i = 1; i < globalArgc; i++) {
	pc = globalArgv[i];
	if (strcmp(pc, "-shell") == 0) {
	    struct stat sbuf;

	    specified_shell = globalArgv[++i];
	    /* was an argument specified? */
	    if (!specified_shell[0]) {
	        Kabort("invalid argument for -shell switch!\n");
	    }
	    /* let's make sure that the specified file exists */
	    if (stat(specified_shell, &sbuf) < 0) {
	        perror(specified_shell);
	        Kabort("invalid argument for -shell switch!\n");
	    }

	    xTrace1(rlp, TR_EVENTS, "RLOGIN: using specified shell %s\n", specified_shell);
	    continue;
	}
        if (strcmp(pc, "-s") == 0) {
	    isserver = 1;
	    continue;
	}
        if (strcmp(pc, "-c") == 0) {
	    isserver = 0;
	    continue;
	}
        if (strcmp(pc, "-v") == 0) {
	    vtapmode = 1;
	    continue;
	}
        if (strcmp(pc, "-n") == 0) {
	    noopenmode = 1;
	    continue;
	}
        if (strcmp(pc, "-noipsec") == 0) {
	    noipsecmode = 1;
	    continue;
	}
        if (strncmp(pc, "-debug", 2) == 0) {
	    pc = globalArgv[++i];
	    if (sscanf(pc, "%d", &debuglevel) != 1) {
	        Kabort("invalid argument for -debug switch!\n");
	    }
	    continue;
	}
    }

    /* initialize all those data structures associated with XObjs */
    protInit(self);

    /* server code */
    if (isserver) {
	/* set up the pipe to the shell */
	if (!noopenmode) {
	    /*
	    server_rhosts_load();
	    */
	    ;
	}

	xTrace0(rlp, TR_GROSS_EVENTS, "RLOGIN init: I am the server!");
	evDetach(evSchedule(server_init, self, 1000));
    } 
    /* client code */
    else {
        Kabort("RLOGIN does not support client mode!");
    }
    xTrace0(rlp, TR_GROSS_EVENTS, "RLOGIN init: done");
}					/* end rlogin_init */


void 
server_init(Event e, void *tself)
{
    Part plist[2];
    XObj self;
    /*
    static long port = IPPORT_LOGINSERVER;
    */
    static long port = LOGIN_PORT;
    PState	*ps;
    XObj rlogin_llp;

    self = (XObj) tself;
    ps = (PState *)self->state;
    xTrace0(rlp, TR_GROSS_EVENTS, "RLOGIN server_init: starting");

    xAssert(xIsXObj(self));
    if (!xIsProtocol(rlogin_llp = xGetDown(self, 0))) {
	Kabort("RLOGIN could not get lower protocol");
    }


    partInit(plist, 1);
    partPush(plist[0], ANY_HOST, 0);
    partPush(plist[0], &port, sizeof(port));

    if (xOpenEnable(self, self, rlogin_llp, plist) != XK_SUCCESS) {
	Kabort("server_init: openenable died...");
    }

    xTrace0(rlp, TR_GROSS_EVENTS, "RLOGIN server_init: xopenable");
    xTrace0(rlp, TR_GROSS_EVENTS, "RLOGIN server_init: finished");
}


static void
sessnInit(s)
    XObj s;
{
    xAssert(xIsSession(s));
      
    s->demux = rloginDemux;
    s->opendone = rlogin_opendone;
}

static void
protInit(p)
    XObj p;
{
    xAssert(xIsProtocol(p));

    p->demux = rloginDemux;
    p->opendone = rlogin_opendone;
    p->open = rloginOpen;
}

static XObj
rloginOpen( self, hlpRcv, hlpType, p )
    XObj	self, hlpRcv, hlpType;
    Part 	*p;
{
    XObj	s, lls;
    PState	*ps = (PState *)self->state;
    
    xTrace0(rlp, TR_MAJOR_EVENTS, "RLOGIN open");
    if ( (lls = xOpen(self, hlpType, xGetDown(self, 0), p)) == ERR_XOBJ ) {
	xTrace0(rlp, TR_ERRORS, "rloginOpen: could not open lower session");
	return ERR_XOBJ;
    }
    if ( mapResolve(ps->activeMap, &lls, &s) == XK_SUCCESS ) {
	xTrace0(rlp, TR_MORE_EVENTS, "rloginOpen -- found an existing one");
    } else {
	xTrace0(rlp, TR_MAJOR_EVENTS, "rloginOpen -- creating a new one");
	if ((s = rloginCreateSession(self, hlpRcv, hlpType, lls)) == ERR_XOBJ) {
	    xClose(lls);
	}
    }
    return s;
}

static XObj
rloginCreateSession(self, hlpRcv, hlpType, lls)
    XObj        self, hlpRcv, hlpType, lls;
{
    XObj s;
    PState *ps = (PState *)self->state;
    SState *ss;

    if ((s = xCreateSessn(sessnInit, hlpRcv, hlpType, self, 1, &lls))
                        == ERR_XOBJ ) {
        xTrace0(rlp, TR_ERRORS, "create sessn fails in rloginCreateSession");
        return ERR_XOBJ;
    }
    s->binding = mapBind(ps->activeMap, &lls, s);
    if ( s->binding == ERR_BIND ) {
        xTrace0(rlp, TR_ERRORS, "mapBind fails in rloginCreateSessn");
        return ERR_XOBJ;
    }
    xSetUp(lls, s);
    ss = X_NEW(SState);
    memset(ss, '\0', sizeof(SState));

    xAssert((ss->buf = malloc(SHELL_BUFFERSZ)) != NULL);

    s->state = (VOID *)ss;

    xDuplicate(lls);
    xTrace1(rlp, TR_MAJOR_EVENTS, "RLOGIN open returns %x", s);
    return s;
}

xkern_return_t 
rlogin_opendone(XObj self, XObj lls, XObj llp, XObj hlpType)
{
    Enable *e;
    PState *ps = (PState *) self->state;
    XObj s;

    if (vtapmode) {
        xControl(lls, VTAP_ENABLETAP, 0, 0);
        xControl(lls, VTAP_PRINTCHARS, 0, 0);
    } else {
        xControl(lls, VTAP_DISABLETAP, 0, 0);
    }

    /* resolve key */
    if (mapResolve(ps->activeMap, &lls, &e) != XK_FAILURE) {
        xTrace0(rlp, TR_MAJOR_EVENTS, "RLOGIN opendone: already in active map");
	return XK_FAILURE;
    }

    /*
    if ((s = rloginCreateSession(self, e->hlpRcv, e->hlpType, lls)) 
    */
    if ((s = rloginCreateSession(self, NULL, NULL, lls)) 
    					== ERR_XOBJ) {
        return XK_FAILURE;
    }
    
    /* type a banner message so we know we're talking to an xkernel thing */
    /*
    {
	char *pstring = "XXX: xkernel rlogin\n";
	Msg msg;
	
	msgConstructEmpty(&msg);
	msgPush(&msg, mycopypush, pstring, strlen(pstring)+1, 0);
	xPush(lls, &msg);
	msgDestroy(&msg);
	
	xControl(lls, TCP_PUSH, 0, 0);
    }   
    */

    xTrace0(rlp, TR_EVENTS,
            "rlogin_opendone: passively opened session successfully created");

    /*  vtap does this */
    return XK_SUCCESS;

    return xOpenDone(e->hlpRcv, s, self);
}


/*
 * how we get a shell is unfortunate.  we create two half-duplex pipes
 * to transmit stdin/stdout to the shell.  we fork.  the child will
 * dup stdin/stdout to the pipe, and then exec("/bin/sh").  the parent
 * will then poll the pipe.
 */

int indexcounter = 0;

void
server_shell_init(XObj s, char * remhost)
{
    int pid;
#ifndef SOLARIS
    int ibuf = 1;
#endif
    SState *ps;

    xTrace0(rlp, TR_GROSS_EVENTS, "RLOGIN server_shell_init: starting");

    xAssert(xIsSession(s));
    ps = s->state;
    ps->index = indexcounter++;

    /* create the two half-duplex pipes */
    if (socketpair(AF_UNIX, SOCK_STREAM, 0, ps->socketfds) < 0) {
        perror("server_shell_init: socketpair()");
	xAssert(0);
    }
#ifndef SOLARIS
    if (setsockopt(ps->socketfds[0], SOL_SOCKET, SO_RCVBUF, (char *) &ibuf, sizeof(int)) < 0) {
        perror("server_shell_init: setsockopt()");
	xAssert(0);
    }
    if (setsockopt(ps->socketfds[1], SOL_SOCKET, SO_RCVBUF, (char *) &ibuf, sizeof(int)) < 0) {
        perror("server_shell_init: setsockopt()");
	xAssert(0);
    }
#endif

    /* fork, and then close the appropriate descriptors */
    if ((pid = fork()) < 0) {
        perror("server_shell_init: fork()");
	xAssert(0);
    }
    
    /* "Advanced Programming in Unix", Richard Stevens, page 429
     *		"For a pipe from the parent to the child, the parent closes
     *		the read end of the pipe (fd[0]) and the child closes the
     *		write end (fd[1])."
     *
     *		i.e., reader is fd[0]
     *		i.e., writer is fd[1]
     *
     * pg 477: 	child uses fd[1]
     *		parent uses fd[0]
     */

#define CHILD 0
#define PARENT 1

    /* if child */

#define CHILDBG
#ifdef CHILDBG				/* normally, you want CHILDBG set */
    if (pid == 0) {
#else
    if (pid) {
#endif
	/* 
	 * The child process needs to be the session leader
	 * and have the pty as its controlling tty.
	 *	(stolen from in.rlogind.c)
	 */

	close(ps->socketfds[PARENT]);

	/* stuff from parent goes to stdin */
	if (ps->socketfds[CHILD] != 0) {
	    if (dup2(ps->socketfds[CHILD], 0) < 0) {
		perror("server_shell_init: dup2()");
		xAssert(0);
	    }
	    /*
	    fprintf(stdout, "RLOGIN: dup2(c,0) okay -- stdin rerouted...\n");
	    */
	}

	/* stdout goes to the parent */
	if (ps->socketfds[CHILD] != 1) {
	    if (dup2(ps->socketfds[CHILD], 1) < 0) {
		perror("server_shell_init: dup2()");
		xAssert(0);
	    }
	    /*
	    fprintf(stdout, "RLOGIN: dup2(c,1) okay -- stdout rerouted...\n");
	    */
	}
	/* stderr goes to the parent */
	if (ps->socketfds[CHILD] != 2) {
	    if (dup2(ps->socketfds[CHILD], 2) < 0) {
		perror("server_shell_init: dup2()");
		xAssert(0);
	    }
	    /*
	    fprintf(stderr, "RLOGIN: dup2(c,2) okay -- stderr rerouted\n");
	    */
	}


	{
	    char argv0[1024], *pcshell;

	    pcshell = specified_shell ? specified_shell : DEFAULT_SHELL_COMMAND;
	    sprintf(argv0, "///---%s", pcshell);

	    /* the hostname (argv[2]) is trusted, authenticated or not */
	    if (!hostname_islegal(remhost) && !noipsecmode) {
		xTrace1(rlp, TR_EVENTS, "RLOGIN server_shell_init: Hostname '%s' not an IP address.  Destroying session...", remhost);
		xDestroy(s);
	    }

	    xTrace2(rlp, TR_EVENTS, "server_shell_init: execing: %s %s\n",
	    		pcshell, argv0);
	    if (specified_shell) 
		execl(pcshell, argv0, NULL);
	    else {
		char * buf[8];
		int i = 0;
		buf[i++] = pcshell;
		if (!noipsecmode) 
		    buf[i++] = "-ipsec";
		if (ps->secflags & SEC_AUTH) 
		    buf[i++] = "-auth";
		if (ps->secflags & SEC_PRIV) 
		    buf[i++] = "-priv";
		buf[i++] = "-n";
		buf[i++] = "-r";
		buf[i++] = remhost;
		buf[i]=NULL;
		execve(pcshell, buf, &buf[i]);
	    }
	}

	/* never should get here */
	Kabort("server_shell_init: execl() failed!");
    }
#ifdef SOLARIS
    (void) setpgrp();		/* setsid */
#else
    (void) setpgrp(0, 0);		/* setsid */
#endif
    /* if parent */
    close(ps->socketfds[CHILD]);

    ps->fd_fromchild = ps->socketfds[PARENT];
    ps->fd_tochild = ps->socketfds[PARENT];

   /* (os) not necessary on Sun
    xTrace0(rlp, TR_EVENTS, "RLOGIN server_shell_init: sleeping...");
    Delay(5000); 
    xTrace0(rlp, TR_EVENTS, "RLOGIN server_shell_init: awaking...");*/

/*  (os) is not needed and will produce an error in the shell since it is fet again 
    in the pipe in the demux routine
    write(ps->fd_tochild, ps->locuser, strlen(ps->locuser)+1);
    write(ps->fd_tochild, ps->remuser, strlen(ps->remuser)+1);
    write(ps->fd_tochild, ps->termtype, strlen(ps->termtype)+1);*/

    xTrace0(rlp, TR_GROSS_EVENTS, "RLOGIN server_shell_init: exiting");
}

int 
hostname_islegal(char *hostname) 
{
    char *pc;
    
    /* check to make sure that the "hostname" is an IP address */
    for (pc = hostname; *pc; pc++) {
	if (*pc != '.' && !isdigit(*pc)) {
	    printf("%s: is not an IP address.  Skipping!\n", hostname);
	    return 0;
	}
    }
    return 1;
}

void 
shell_poll (Event e, void *vself)
{
    XObj self = (XObj) vself;
    int nread;
    static int nfds, firsttime = 0;
    static struct timeval timeout;
    XObj lls;
    SState *ps;

    if (!xIsSession(lls = xGetDown(self, 0))) {
	Kabort("RLOGIN could not get lower session");
    }

    ps = self->state;

    /* only call once */
    if (firsttime == 0) {
#ifndef SOLARIS
	nfds = getdtablesize();
#else
	nfds = 64;
#endif
	timeout.tv_sec = 0;
	timeout.tv_usec = 0;
    }

    for (;;) {
	FD_ZERO(&ps->readfds);
	FD_SET(ps->fd_fromchild, &ps->readfds);

	RDEBUG(10) xTrace1(rlp, TR_EVENTS, "RLOGIN shell_poll (session %d): polling...", ps->index);

	if (select(nfds, &ps->readfds, (fd_set *) 0, (fd_set *) 0, &timeout) < 0) {
	    perror("shell_poll: select()");
	    Kabort("shell_poll: select() failed!");
	}

	/* is the shell saying something? */
	if (!FD_ISSET(ps->fd_fromchild, &ps->readfds)) {
	    Delay(5);
	    continue;
	}

	/* non-zero return expected */
	if ((nread = read(ps->fd_fromchild, ps->buf, SHELL_BUFFERSZ)) < 0) {
	    perror("fread()");
	    Kabort("shell_poll: read() failed!");
	}
	if (nread == 0)
	{
	    xClose(lls);
	    mapRemoveBinding(((PState *)self->myprotl->state)->activeMap, self->binding);
	    xTrace0(rlp, TR_EVENTS, "RLOGIN shell died close session...");
	    return;
	}

	RDEBUG(10) xTrace2(rlp, TR_EVENTS, "RLOGIN shell_poll (session %d): read %d bytes...", ps->index, nread);

	RDEBUG(100) xTrace0(rlp, TR_EVENTS, "RLOGIN shell_poll: started xPush...");

	msgConstructBuffer(&ps->msg_shell, ps->buf, nread);
	xPush(lls, &ps->msg_shell);
	msgDestroy(&ps->msg_shell);

	xControl(lls, TCP_PUSH, 0, 0);

	RDEBUG(10) xTrace0(rlp, TR_EVENTS, "RLOGIN shell_poll: finished xPush...");
    }
}

XObj rlogin_lls;


static xkern_return_t 
rloginDemux(XObj self, XObj lls, Msg * msg)
{
    int len, len2;
    char buf[2048];
    char buf2[1024];
    static char *pcin, *pcout;
    int numc;
    int retval;
    SState *ps;
    char * remhost;

    xTrace0(rlp, TR_MORE_EVENTS, "RLOGIN CallDemux");

    if (!xIsSession(self)) {
	Kabort("RLOGIN could not get lower session");
    }
    ps = self->state;

    len = msgLen(msg);
    msgPop(msg, mycopypop, buf, len, 0);
    xTrace1(rlp, TR_GROSS_EVENTS, "RLOGIN demux: received writer port (len=%d)!", len);

    /* is this the first demux for the session?  if so, we need to fork off
     * a shell based on the information given by rcmd.
     */
    xTrace2(rlp,TR_EVENTS, "rloginDemux: ENTER: state=%d (len=%d)\n", ps->initialized, len);

    /* state machine:
    * 
    *     1.  we need to get a null from the client
    *     2.  we need to get three null terminated strings from the client
    *     	a.  locuser
    * 		b.  remuser
    * 		c.  termtype
    */

    /* remember, it's (numc <= len) because we use the last trailing null 
     * in the string.
     */
    if (ps->initialized != 4 && len > 1) {
	for (pcin = buf, numc = 0; numc <= len; pcin++, numc++) {
	    switch (ps->initialized) {
	    case 0:
		ps->initialized = 1;
		ps->locuser = malloc(512);
		ps->remuser = malloc(512);
		ps->termtype = malloc(512);

		pcout = ps->locuser;

	    /* make sure that we're running on top of ipsec */
	    {
		IPhost remaddress;
		int err;

		if ((err = xControl(lls, IPSEC_GETPEERHOST, (char *)&remaddress,
				    sizeof(IPhost))) == -1) {
		    xTrace1(rlp, TR_EVENTS, 
			"RLOGIN xControl IPSEC_GETPEERHOST returned %d\n", err);
		    if (xControl(lls, GETPEERHOST, (char *)&remaddress,
				    sizeof(IPhost)) == -1)
			Kabort("RLOGIN can't determin peer address");	    			
		} else
		    ps->secflags |= SEC_AUTH;
		remhost = xMalloc(strlen(ipHostStr(&remaddress))+1);
		strcpy(remhost,ipHostStr(&remaddress));
	    }
	    {
		int err;

		if ((err = xControl(lls, CRYPT_ISPRIVATE, 0, 0)) != 1) {
		    xTrace1(rlp, TR_EVENTS, 
			"RLOGIN xControl CRYPT_ISPRIVATE returned %d\n", err);
		} else
		    ps->secflags |= SEC_PRIV;
	    }

					/* fall through */
	    case 1:
		if (*pcin) {
		    *pcout = *pcin;
		     pcout ++;
		}
		else {
		    *pcout = *pcin;
		    pcout = ps->remuser;
		    ps->initialized = 2;
		}
		break;
	    case 2:
		if (*pcin) {
		    *pcout = *pcin;
		     pcout ++;
		}
		else {
		    *pcout = *pcin;
		    pcout = ps->termtype;
		    ps->initialized = 3;
		}
		break;
	    case 3:
		if (*pcin) {
		    *pcout = *pcin;
		     pcout ++;
		}
		else {
		    *pcout = *pcin;
		    xTrace4(rlp,TR_EVENTS,"locuser=%s, remuser=%s, termtype=%s remot_host_addr=%s\n",
			    ps->locuser, ps->remuser, ps->termtype, remhost);
		    ps->initialized = 4;

		    /* assign a shell to the session */
		    server_shell_init(self, remhost);
		    evDetach(evSchedule(shell_poll, self, 0));

		    /* send null message to tell client that we succeeded */
		    {
			char *pstring = "";
			
			msgConstructEmpty(&ps->msg_shell);
			msgPush(&ps->msg_shell, mycopypush, pstring, 1, 0);
			xPush(lls, &ps->msg_shell);
			msgDestroy(&ps->msg_shell);

			xControl(lls, TCP_PUSH, 0, 0);
		    }
		}
		break;
	    case 4:		/* by this time, we've forked the shell */
		break;
	    default:			/* error */
		xTrace1(rlp,TR_EVENTS, "RLOGIN rloginDemux: unknown state %d!\n", 
		    ps->initialized);
		xAssert(0);
	    }
	}
    }

    if (ps->initialized != 4) {
	xTrace2(rlp,TR_EVENTS,"rloginDemux: EXIT: state=%d (len=%d)\n", ps->initialized, len);
	return XK_SUCCESS;
    }

    /* now feed the incoming message into the shell pipe */

    /* first let's try to emulate the streams interface
     *		CR -> CR + NL
     */
    {
	char *p1, *p2;
	int i;

	len2 = 0;		/* keep revised count of xformed string */

	for (p1 = buf, p2 = buf2, i = 0; i < len; i++) {
	    switch (*p1) {
	    case '\015': 
	    	*p2++ = *p1++;
		*p2++ = '\012';
		len2 += 2;
		break;
	    default:
	        *p2++ = *p1++;
		len2++;
	    }
	}
    }

    if ((retval = write(ps->fd_tochild, buf2, len2)) != len) {
	perror("write()");
	fprintf(stderr, "rloginDemux: write() returned %d (expected %d)!\n",
				retval, len2);
	Kabort("rloginDemux: write() failed!");
    }

    xTrace0(rlp, TR_MORE_EVENTS, "RLOGIN serverdemux exiting...");

    return XK_SUCCESS;
}

#define TCPBUFSIZ 4096

/*
 * string_split_space(char *string, char *s, char *t)
 *
 * 	given (string), place the first word into (s), and the rest of
 *	into (t).
 *
 *	returns zero on success, -1 on failure.
 */

int
string_split_space (char *string, char *s, char *t)
{
    char *sp;

    /*
     * (char *sp) = the first space.  	s = {string[0..(sp-s-1)]}
     *			      		t = {sp[1..end]}
     */

    if ((sp = strchr(string, ' ')) == NULL) {
	if ((sp = strchr(string, '\t')) == NULL) {
	    return -1;
	}
    }

    /* don't forget to null-terminate the string w/strncpy() */
    (void) strncpy(s, string, sp-string);
    s[sp-string] = '\0';

    (void) strcpy(t, sp+1);
    return 0;
}

