/*#include <sys/systeminfo.h>*/
#include <sys/types.h>
#include <time.h>
#include <stdlib.h>
#include "xkernel.h"
#include "tcp.h"
#include "ip.h"
#include "udp.h"
#include "x_stdio.h"
#include "tcpf_internal.h"
#include "rc.h"

#define HOST_TYPE IPhost
#define BUFFER_SIZE 4096

#define TRACE_VAR tcpf_sp
#define TCP_NAME "tcp"
#define MAP_SIZE 101
#define INIT_SEQ 1

typedef struct {
  Protl tcp_protl, link_protl;
  Sessn link_sessn;
  Map   bindmap;
  u_int next_CID;
} PState;

typedef enum { 
  TCPF_INACTIVE, TCPF_OPENING, TCPF_CLOSING, TCPF_OK 
} ConnectStatus;

typedef struct {
  ConnectStatus  status;
  int SourceAddr, DestAddr;
  long SourcePort, DestPort;
  u_int CID, seq, turn;
} CState;

int tracetcpf_sp;

static xkern_return_t   senderDemux( Protl, Sessn, Msg * );
static xkern_return_t   openDone( Protl, Protl, Sessn, Protl );
static xkern_return_t   closeDone( Sessn );
static xkern_return_t   customDemux( Protl, Sessn, Msg * );
static xkern_return_t   customOpenDone( Protl, Protl, Sessn, Protl );
static void             identifyDownProtocols ( Protl );
static int              IPhost2int (IPhost *);

int tcpf_s_init (Protl);

int tcpf_s_init (self)
     Protl self;
{
  PState	*ps;
  Part lp;
  
  printf ("I am the send-side router\n");
  
  /* Initialize protocol method table */
  self->demux = senderDemux;
  self->opendone = openDone;
  self->closedone = closeDone;

  ps = X_NEW(PState);
  bzero((char *)ps, sizeof(PState));
  self->state = (VOID *)ps;

  identifyDownProtocols (self);
  /*  tcpfProcessRomFile(self); */
  str2ipHost(&tcpfRcvrAddr, "192.12.71.2");
  tcpfRcvrPort = 999;
  
  ps->next_CID = rand();
  ps->bindmap = mapCreate (MAP_SIZE, sizeof (Sessn));
  
  /* Open single session over one-way link */
  partInit (&lp, 1);
  partPush(lp, &(tcpfRcvrAddr), sizeof(HOST_TYPE));
  partPush(lp, &(tcpfRcvrPort), sizeof(LinkPort));
  
  ps->link_sessn = xOpen(self, self, ps->link_protl, &lp);
  
  if ( ps->link_sessn == ERR_SESSN ) 
    {
      Kabort ("TCPF_S can't open link protocol");
    } 

  return 0;
}


static void identifyDownProtocols (self)
     Protl self;
{
  Protl llp1, llp2, tcp;
  PState	*ps = (PState *)self->state;
  
  xTrace1(TRACE_VAR, TR_EVENTS, "Using \"%s\" for TCP name", TCP_NAME);

  tcp = xGetProtlByName (TCP_NAME);
  if (!xIsProtocol(tcp)) {
      Kabort ("Could not find a protocol object for TCP");
    }
  
  llp1 = xGetDown(self, 0);
  llp2 = xGetDown(self, 1);
  if (!xIsProtocol(llp1) || !xIsProtocol(llp2)) {
      Kabort("TCPF_S does not have two lower protocols");
    }

  if (llp1 == tcp)
    {
      ps->tcp_protl = llp1;
      ps->link_protl = llp2;
    }
  else if (llp2 == tcp)
    {
      ps->link_protl = llp1;
      ps->tcp_protl = llp2;
    }
  else
    {
      Kabort("TCPF_S is not configured over TCP");
    }
}


static
#if defined(__GNUC__)
inline
#endif
void checkHandle( h, str )
    xmsg_handle_t	h;
    char		*str;
{
    switch ( h ) {
      case XMSG_ERR_HANDLE:
      case XMSG_ERR_WOULDBLOCK:
	sprintf(errBuf, "%s returns error handle %d", str, h);
	Kabort(errBuf);
      default:
	;
    }
}


/*
 * Receive a message from a TCP connection and forward it out on the
 * one-way link.
 */
xkern_return_t
senderDemux( self, lls, dg )
    Protl self;
    Sessn lls;
    Msg *dg;
{
  PState	*ps = (PState *)self->state;
  CState *cs;
  static int c = 1;
  struct DataHdr hdr;
  char   *buf;
  u_short m;
  struct rcinfo  i;
  
  xIfTrace(TRACE_VAR, TR_MAJOR_EVENTS) 
    {
      putchar('.');
      if (! (c++ % 50)) 
	{
	  putchar('\n');
	}
    } 
  
  /* Retrieve state from map based on LLS */
  if (mapResolve(ps->bindmap, &lls, (void **)&cs) == XK_FAILURE) {
      xTrace0 (TRACE_VAR, TR_ERRORS, 
	       "received message belongs to no known connection");
      return XK_FAILURE;
    }
 
  m = msgLen(dg);
  
  /* Add header */ 
  hdr.type = htonl(DATA_MSG);
  hdr.CID = htonl(cs->CID);
  hdr.seq = htonl(cs->seq++);
  memcpy(buf + sizeof(struct DataHdr), dg, sizeof(Msg));
  memcpy(buf, &hdr, sizeof(struct DataHdr));
      
  /* Add message attribute, increment ref cnt on lls */
  i.tcp_s = lls;
  xDuplicate (lls);
  i.len = m;
  msgSetAttr (dg, 0, (void *) &i, sizeof (struct rcinfo));
      
  xPush (ps->link_sessn, dg);
  
  return XK_SUCCESS;
}


static xkern_return_t
openDone( self, llp, s, hlpType )
    Protl self, llp, hlpType;
    Sessn s;
{
  PState  *ps = (PState *)self->state;
  CState  *cs;
  u_short space = BUFFER_SIZE;
  struct OpenHdr hdr;
  Msg     m;
  char    *buf;
  IPpseudoHdr  IPhdr;
  
  xTrace0(TRACE_VAR, TR_MAJOR_EVENTS, "TCPF_S openDone");

  if (mapResolve (ps->bindmap, &s, (void **) &cs) == XK_SUCCESS) {
      if (cs->status != TCPF_OPENING)
	{
	  xTrace0 (TRACE_VAR, TR_MAJOR_EVENTS, 
		   "openDone: binding already exists");
	  return XK_FAILURE;
	}
      /*
       * This xDuplicate() is needed under xkernel 3.3 but not 3.2
       */
      xDuplicate(s);
      return XK_SUCCESS;
    }
  
  xDuplicate(s);

  /* create appropriate state and enter into map */
  cs = X_NEW (CState);
  cs->seq = INIT_SEQ;
  cs->CID = ps->next_CID++;
  cs->status = TCPF_OPENING;
  
  /* query lls to find out IP addresses */
  xControl(s, IP_GETPSEUDOHDR, (char *) &IPhdr, sizeof(IPhdr));
  
  cs->SourceAddr = IPhost2int(&IPhdr.dst);
  xControl(s, GETPEERPROTO, (char *) &(cs->SourcePort), sizeof(long));;
  cs->DestAddr = IPhost2int(&IPhdr.src);

  xControl(s, GETMYPROTO, (char *) &(cs->DestPort), sizeof(long));;
   
  if ( mapBind (ps->bindmap, (void *) &s, (void *) cs) == ERR_BIND)
    {
      xTrace0 (TRACE_VAR, TR_ERRORS, "openDone: binding failed");
      return XK_FAILURE;
    }
  
  xTrace1 (TRACE_VAR, TR_EVENTS, "Opening connection, CID %d", cs->CID);
  xTrace5 (TRACE_VAR, TR_EVENTS, "Source IP addr: %d.%d.%d.%d, TCP port: %d",
	   IPhdr.dst.a, IPhdr.dst.b, IPhdr.dst.c, IPhdr.dst.d,
	   cs->SourcePort);
  xTrace5 (TRACE_VAR, TR_EVENTS, "Dest IP addr: %d.%d.%d.%d, TCP port: %d",
	   IPhdr.src.a, IPhdr.src.b, IPhdr.src.c, IPhdr.src.d,
	   cs->DestPort);
  
  /* send an open message out on link */
  hdr.type = htonl(OPEN_MSG);
  hdr.CID = htonl(cs->CID);
  hdr.seq = htonl(cs->seq++);
  hdr.SourceAddr = cs->SourceAddr;
  hdr.SourcePort = htonl(cs->SourcePort);
  hdr.DestAddr = cs->DestAddr;
  hdr.DestPort = htonl(cs->DestPort);
  
  msgConstructAllocate (&m, sizeof (struct OpenHdr), &buf);
  memcpy (buf, &hdr, sizeof (struct OpenHdr));

  /* set TCP buffeer size */
  if (xControl(s, TCP_SETRCVBUFSIZE, (char*)&space, sizeof(space)) < 0) 
    xError("saveServerSessn: TCP_SETRCVBUFSIZE failed");
  if (xControl(s, TCP_SETRCVBUFSPACE, (char*)&space, sizeof(space)) < 0)
    xError("saveServerSessn: TCP_SETRCVBUFSPACE failed");

  xPush (ps->link_sessn, &m);

  cs->status = TCPF_OK;
  
  return XK_SUCCESS;
}


static xkern_return_t
closeDone( lls )
    Sessn	lls;
{
  PState	*ps;
  CState        *cs;
  Protl          self;
  struct CloseHdr hdr;
  char          *buf;
  Msg           m;
  
  printf ("closeDone called\n");
  
  /* Retrieve state from map based on LLS */
  self = xGetUp (lls);
  ps = (PState *)self->state;
  if (mapResolve(ps->bindmap, &lls, (void **)&cs) == XK_FAILURE) {
      xTrace0 (TRACE_VAR, TR_ERRORS, 
	       "close belongs to no known connection");
      return XK_FAILURE;
    }

  if (cs->status == TCPF_CLOSING)
    return XK_SUCCESS;
  
  cs->status = TCPF_CLOSING;
  
  /* Send "Close" message out on link for CID */
  hdr.type = htonl(CLOSE_MSG);
  hdr.CID = htonl(cs->CID);
  hdr.seq = htonl(cs->seq++);
  
  msgConstructAllocate (&m, sizeof (struct CloseHdr), &buf);
  memcpy (buf, &hdr, sizeof (struct CloseHdr));
  
  xPush (ps->link_sessn, &m);

  /* Remove state from map */
  if ( mapUnbind (ps->bindmap, &lls) == XK_FAILURE )
    {
      xTrace0 (TRACE_VAR, TR_ERRORS, "mapUnbind failed");
    }

  xClose (lls);
  
  free (cs);
  
  return XK_SUCCESS;
}


static xkern_return_t
customDemux( self, lls, dg )
    Protl	self;
    Sessn	lls;
    Msg		*dg;
{
  u_short space = BUFFER_SIZE;

  xTrace1(TRACE_VAR, TR_DETAILED, "TCP custom demux resets buffer to size %d",
	  BUFFER_SIZE);
  if (xControl(lls, TCP_SETRCVBUFSPACE, (char*)&space, sizeof(space)) < 0) 
    xError("customDemux: TCP_SETRCVBUFSPACE failed");
  
  return XK_SUCCESS;
}


static int IPhost2int (IPhost *x)
{
  int y;
  
  y = x->a << 24;
  y |= x->b << 16;
  y |= x->c << 8;
  y |= x->d;

  return y;
}


