/*
 * $RCSfile: icmptest.c,v $
 *
 * x-kernel v3.2
 *
 * Copyright (c) 1993,1991,1990  Arizona Board of Regents
 *
 *
 * $Log: icmptest.c,v $
 * Revision 1.26  1996/05/30 04:36:26  spatsch
 * Fixed bugs... .
 *
 * Revision 1.25  1995/12/01  20:38:53  ho
 * Control op name change.
 *
 * Revision 1.24  1993/12/16  01:49:02  menze
 * Fixed type of 'client' function to conform to event interface
 *
 */

/*
 * This protocol is a ping-pong test for the ICMP echo request and reply
 * messages.
 *
 * The same protocol code is used for both the client and the server.  
 * (The server is null since ICMP automatically sends the echoes.)
 */

#include "xkernel.h"
#include "trace.h"
#ifndef XKMACHKERNEL
#include "x_stdio.h"
#endif /* ! XKMACHKERNEL */
#include "icmp.h"


#define HOST_TYPE IPhost
#define INIT_FUNC dummy_init
#define TRACE_VAR traceicmptestp
#define PROT_STRING "icmp"

/* 
 * If a host is booted without client/server parameters and matches
 * one of these addresses, it will come up in the appropriate role.
 */
static IPhost ServerAddr = { 0, 0, 0, 0 };
static IPhost ClientAddr = { 0, 0, 0, 0 };

    static int lens[] = { 
	1, 1000, 2000, 4000, 8000, 16000
      };

#define TRIPS 10
#define TIMES 2
#define DELAY 3

/*
 * Define to do timing calculations
 */
#define TIME

#include "common_test.c"

static XObj myProtl;
static int count;

#ifdef __STDC__

static void	icmpclient( Event, void * );
int 	icmptest_init(XObj);

#else

static void	icmpclient();
int 	icmptest_init();

#endif


int icmptest_init( self )
    XObj self;
{
    XObj	llp;
    PState	*ps;

    xIfTrace(prottest, TR_ERRORS) { print_compile_options(); };
    processOptions();
    if ( testsSpecified && ! myTestSpecified ) {
	xTrace1(prottest, TR_SOFT_ERRORS, "Parameters indicate %s test should not run",
		PROT_STRING);
	return 0;
    }
    printf("%s timing test\n", PROT_STRING);
    llp = xGetDown(self, 0);
    if ( ! xIsProtocol(llp) ) {
	Kabort("Test protocol has no lower protocol");
    }
    xControl(xGetDown(self, 0), GETMYHOST, (char *)&myHost, sizeof(HOST_TYPE));
    ps = X_NEW(PState);
    bzero((char *)ps, sizeof(PState));
    self->state = (VOID *)ps;
    /* 
     * Call the per-test initialization function which gives the test
     * the opportunity to override the default functions
     */
     myProtl=self;
    if ((*isServer)(self)) {
	return XK_SUCCESS;
    } else if ( (*isClient)(self)) {
#ifdef CONCURRENCY	
	int	i;

	for ( i=0; i < CONCURRENCY; i++ )
#endif
	    evDetach( evSchedule(icmpclient, self, 0) );
    } else {
	printf("%stest: I am neither server nor client\n", PROT_STRING);
    }
    return XK_SUCCESS;
}

static void
testInit()
{
}


static void
icmpclient( ev, arg )
    Event	ev;
    VOID	*arg;
{
    XObj s;
    Part p;
    int test = 0;
#ifdef XK_DEBUG
    int	total = 0;
#endif
    int lenindex, len, i;
        
    s = NULL;
    printf("I am the client, talking to <%s>\n", ipHostStr(&ServerAddr));
    
    partInit(&p, 1);
    partPush(p, &ServerAddr, sizeof(IPhost));
    
    if ((s = xOpen(myProtl, myProtl, xGetDown(myProtl, 0), &p)) == ERR_XOBJ) {
	xTrace0(prottest, TR_ERRORS, "Not sending, could not open session");
	return;
    } else {
#ifdef INF_LOOP
	for (lenindex = 0; ; lenindex++) {
	    if (lenindex >= sizeof(lens)/sizeof(long)) lenindex = 0;
#else
	for (lenindex = 0; lenindex < sizeof(lens)/sizeof(long); lenindex++) {
#endif
	    len = lens[lenindex];
	    for (test = 0; test < TIMES; test++) {
		xTrace2(prottest, TR_MAJOR_EVENTS, "Sending (%d)  len = %d ...\n",
			++total, len);
		for (i=0; i < TRIPS; i++) {
		    if (xControl(s, ICMP_ECHO_CTL, (char *)&len,
				 sizeof(len))) {
			xTrace1(prottest, TR_ERRORS, "\nfailed after %d trips", i);
			break;
		    }
		    xIfTrace(prottest, TR_MAJOR_EVENTS) {
			putchar('.');
			if (! (++count % 50)) {
			    putchar('\n');
			}
		    }
		}
		xTrace1(prottest, 1, "\n%d round trips completed", i);
		Delay(DELAY * 1000);
	    }
	}
#ifdef ABORT
        Kabort("End of test");
#endif
        return;
    }
}