/*
 *
 * x-kernel v3.2
 *
 * Copyright (c) 1993,1991,1990  Arizona Board of Regents
 *
 */

#include "xkernel.h"
#include "mcastroute.h"

#ifdef __STDC__
 
static mcastRoute *mcast_rt_alloc(mcastRouteTable *);
static void        mcast_rt_free(mcastRouteTable *, mcastRoute *);
static int         mcast_rt_hash(IPhost *);
static mcastRoute *mcast_rt_new(mcastRouteTable *, IPhost *, ifcListType *);
 
#else
 
static mcastRoute *mcast_rt_alloc();
static void        mcast_rt_free();
static int         mcast_rt_hash();
static mcastRoute *mcast_rt_new();
 
#endif /* __STDC__ */

extern tracevnetp;

xkern_return_t
mcast_rt_init(ps) 
    PState *ps;
{
    mcastRouteTable *tbl = &ps->mRtTbl;

    xTrace0 (vnetp, TR_GROSS_EVENTS, "VNET mcast_rt_init()");
    tbl->arr = (mcastRoute **)xMalloc(MCAST_RT_TABLE_SZ * sizeof(mcastRoute *));
    tbl->bpoolsize = MBPSIZE;
    return XK_SUCCESS;

}

static mcastRoute *
mcast_rt_alloc(tbl)
    mcastRouteTable *tbl;
{
    if ( tbl->bpoolsize == 0 ) {
        xTrace0(vnetp, TR_SOFT_ERRORS, "vnet mcast_rt_alloc ... multicast route table is full");
        return 0;
    }
    tbl->bpoolsize--;
    return (mcastRoute *)xMalloc(sizeof(mcastRoute));
}

static int
mcast_rt_hash(address)
    IPhost *address;
{
    u_short sum;

    sum	= address->a + address->b + address->c + address->d;
    return (sum % MCAST_RT_TABLE_SZ);
}

static mcastRoute *
mcast_rt_new(tbl, address, ifclist)
    mcastRouteTable *tbl;
    IPhost *address;
    ifcListType *ifclist;
{
    mcastRoute *rtPtr;

    rtPtr = mcast_rt_alloc (tbl);
    if (rtPtr) {
        rtPtr->mcastAddress = *address;
        rtPtr->ifcList = *ifclist;
        rtPtr->next = NULL;
    }
    return rtPtr;
}


xkern_return_t
mcast_rt_add( pstate, address, ifclist)
    PState *pstate;
    IPhost *address;
    ifcListType *ifclist;
{
    mcastRoute *posnPtr, *prevPtr, *rtPtr;
    u_short hashvalue;
    mcastRouteTable *tbl = &pstate->mRtTbl;


    rtPtr = mcast_rt_new ( tbl, address, ifclist);
    if (rtPtr == 0) 
        return XK_FAILURE;

    hashvalue = mcast_rt_hash(address);
    posnPtr = tbl->arr[hashvalue];
    prevPtr = NULL;
    while (posnPtr != NULL) {
        if (IP_EQUAL(posnPtr->mcastAddress, *address) )
            break;
        prevPtr = posnPtr;
        posnPtr = posnPtr->next; 
    }
    if (posnPtr != NULL) {
        /* route for the address already exists, replace it */
        rtPtr->next = posnPtr->next;
        mcast_rt_free(tbl, posnPtr);
    }
    if (prevPtr == NULL) 
        tbl->arr[hashvalue] = rtPtr;
    else 
        prevPtr->next = rtPtr;
    return XK_SUCCESS;
}

static void
mcast_rt_free(tbl, rt)
    mcastRouteTable *tbl;
    mcastRoute *rt;
{
    tbl->bpoolsize++;
    xFree((char *)rt);
}

xkern_return_t
mcast_rt_get(tbl, address, req)
    mcastRouteTable *tbl;
    IPhost *address;
    mcastRoute *req;
{
    mcastRoute *posnPtr;
    u_short hashvalue;
 
    hashvalue = mcast_rt_hash(address);
    posnPtr = tbl->arr[hashvalue];
    while (posnPtr != NULL) {
        if (IP_EQUAL(posnPtr->mcastAddress, *address) )
            break;
        posnPtr = posnPtr->next;
    }
    if (posnPtr == NULL)
        return XK_FAILURE;
    else {
        *req = *posnPtr;
        return XK_SUCCESS;
    }
}
