%{
#include <stdio.h>
#include <string.h>

typedef struct _integer_t {
    unsigned long val;
    char *str;
} integer_t;
 
#include "y.tab.h"

#ifdef FLEX_SCANNER
extern char *yytext;
#endif
%}
 
%%

[ \t\n]+	;

"="	  return('=');
";"	  return(';');

spi	|
SPI	  return(SPI);
host	|
HOST	  return(HOST);
alg	|
ALG	  return(ALG);
iv	|
IV	  return(IV);
mode	|
MODE	  return(MODE);
key	|
KEY	  return(KEY);
key_len	|
KEY_LEN	  return(KEY_LEN);

md5	|
MD5	  return(MD5);
des	|
DES	  return(DES);
des3	|
DES3	  return(DES3);
des3in	|
DES3IN	  return(DES3IN);
rsasign |
RSASIGN   return(RSASIGN);

ah	|
AH	  return(AH);
esp	|
ESP	  return(ESP);
esptunnel	|
ESPTUNNEL	  return(ESPTUNNEL);

local	|
LOCAL	  return(LOCAL);
remote	|
REMOTE	  return(REMOTE);

0x[0-9a-fA-F]+	|
0X[0-9a-fA-F]+	  { int i;
		    yylval.integer.val = 0;
		    if (yyleng > 10) yyleng = 10;	/* ignore the ending digits */
		    for (i = 2; i < yyleng; i++) {	/* caculate the value */
			if (yytext[i] >= '0' && yytext[i] <= '9')
			    yylval.integer.val = yylval.integer.val * 16 + yytext[i] - '0';
			else if (yytext[i] >= 'a' && yytext[i] <= 'f')
			    yylval.integer.val = yylval.integer.val * 16 + yytext[i] - 'a' + 10;
			else 
			    yylval.integer.val = yylval.integer.val * 16 + yytext[i] - 'A' + 10;
		    }

		    yylval.integer.str = (char *)malloc((strlen(yytext) + 1) > 49 ? (strlen(yytext) + 1) : 49);
		    strcpy(yylval.integer.str, yytext + 2);
#ifdef DEBUG
printf("%x\n", yylval.integer.val);
printf("%s\n\n", yylval.integer.str);
#endif

		    return(INTEGER);
		  }

0[0-7]+		  { int i, a, b, c, d, tmp;
		    char str1[49], str2[49];

		    str1[0] = str2[0] = '\0';
		    if (yyleng > 32/3+1) yyleng = 11;	/* ignore the ending numbers */
		    yylval.integer.val = 0;
		    for (i = 1; i < yyleng; i++) 
		      yylval.integer.val = yylval.integer.val * 8 + yytext[i] - '0';

		    /* here I changed the basis 8 representation to hex string */
		    for (i = yyleng - 1; i >= 1; i = i - 4) { 
		      /* from end to beginning, process 4 chars(3 hex digits) each time */
			a = (i   >= 1 ? yytext[i]   - '0': 0);
			b = (i-1 >= 1 ? yytext[i-1] - '0': 0);
			c = (i-2 >= 1 ? yytext[i-2] - '0': 0);
			d = (i-3 >= 1 ? yytext[i-3] - '0': 0);
			
			tmp = a + b*8 + c*64 + d*512;
			sprintf(str1, "%03x", tmp);
			strcat(str1, str2); strcpy(str2, str1);
		    }
		    for (i = 0; str2[i] == '0'; i++) ; /* delete the leading zeros */
		    yylval.integer.str = (char *)malloc((strlen(str2 + i) + 3) > 49 ? (strlen(str2 + i) + 3) : 49);
		    strcpy(yylval.integer.str, str2+i);
		    if (strlen(yylval.integer.str) % 2)
		      strcat("0", yylval.integer.str);  /* add leading zero to make even length */
		    /* yylval.integer.str[strlen(str2+i)] = '\0'; */
#ifdef DEBUG
printf("%o\n", yylval.integer.val);
printf("%s\n\n", yylval.integer.str);
#endif

		    return(INTEGER);
		  }

[1-9][0-9]*	  { int i;
		    yylval.integer.val = 0;
		    if (yyleng > 12) yyleng = 12;
		    for (i = 0; i < yyleng; i++) 
			yylval.integer.val = yylval.integer.val * 10 + yytext[i] - '0';

		    /* how to transfer decimal rep. of numbers to hex rep. ? */
		    yylval.integer.str = (char *)malloc(49);
		    sprintf(yylval.integer.str, "%x", yylval.integer.val);
#ifdef DEBUG
printf("%u\n", yylval.integer.val);
printf("%s\n\n", yylval.integer.str);
#endif

		    return(INTEGER);
		  }

0+		  { int i;
		    yylval.integer.val = 0;
		    yylval.integer.str = (char *)malloc(49);
		    yylval.integer.str[0]= '0';
		    yylval.integer.str[1]= '\0';
#ifdef DEBUG
printf("%u\n", yylval.integer.val);
printf("%s\n\n", yylval.integer.str);
#endif

		    return(INTEGER);
		  }

[1-9][0-9]*\.[1-9][0-9]*\.[1-9][0-9]*\.[1-9][0-9]*  { /* Maybe it is better to treat IPAddr as Integer */
		    int i, j;
		    for (j = 0; j < 4; j++)
			yylval.ipaddr[j] = 0;
		    for (i = 0, j = 0; i < yyleng; i++)
			if (yytext[i] == '.') j++;
			else yylval.ipaddr[j] = yylval.ipaddr[j] * 10 + yytext[i] - '0';
#ifdef DEBUG
for (i = 0; i < 4; i++) {
    printf("%d\t", yylval.ipaddr[i]);
}
printf("\n\n");
#endif
		    return(IPADDR);
		  				    }

\'[^\"]+\'	|
\"[^\"]+\"  	{  int i;
		   if (yyleng > 6) yyleng = 6;
		   yylval.integer.val = 0;
		   for (i = 1; i < yyleng - 1; i++) {
		       yylval.integer.val = yylval.integer.val * 256 + yytext[i];
		   }
		   yylval.integer.str = (char *)malloc((2 * strlen(yytext)) > 49 ? (2 * strlen(yytext)) : 49);
		   for (i = 1; i < strlen(yytext) - 1; i++)
		     sprintf(yylval.integer.str+2*(i-1), "%02x", yytext[i]);
		   yylval.integer.str[2*(i-1)] = '\0';
#ifdef DEBUG
printf("%x\n", yylval.integer.val);
printf("%s\n\n", yylval.integer.str);
#endif

		   return(INTEGER);
		}

\/\*[^\/\*]*\*\/	  ;


[^ \t\n;=\"\']+	{  fprintf(stderr, "Illegal word %s--Ignored\n", yytext);
		}
%%

