/*
 * $RCSfile: expr_defs.h,v $
 *
 * Universal Stub Compiler
 *
 * Copyright (c) 1994    Arizona Board of Regents
 *
 * $Log: expr_defs.h,v $
 * Revision 1.1  1994/10/13 17:43:30  dorgival
 * Initial revision
 *
 */

#ifndef _INEXPRDEFSH
#define _INEXPRDEFSH

typedef enum { 
  CONST_EXPR, 
  OP_EXPR, 
  SYMINFO_EXPR, 
  IDENT_EXPR
} ExprType;


typedef struct bin_op {
  int theoper;      /* the actual lex token */
  struct expr *left;
  struct expr *right;
} BinOp;

typedef struct {
  int num;
  Symbol thesym;
  Symbol basesym;
  Symbol last_struct;
  TypeData *thedata;
  struct expr *structs_offset;   /* offset of last_struct */
  struct expr *offset_from_base;
} SymInfo;

#define SYMINFO_SYM(a)        (&((a)->thesym))
#define SYMINFO_BASESYM(a)    (&((a)->basesym))
#define SYMINFO_LASTSTRUCT(a) (&((a)->last_struct))
#define SYMINFO_TYPEDATA(a)   ((a)->thedata)
#define SYMINFO_OFFSET(a)     ((a)->offset_from_base)
#define SYMINFO_STRUCTOFFSET(a) ((a)->structs_offset)
#define SYMINFO_NUM(a)        ((a)->num)

typedef struct expr{
  ExprType thetype;
  union {
    Constant *theconst;
    BinOp *theopr;
    SymInfo *thesyminfo;
    char *theident;
  } u;
  struct expr* next;
} Expr;

#define EXPR_TYPE(a)         ((a)->thetype)
#define EXPR_CONST(a)        ((a)->u.theconst)
#define EXPR_CONST_UNIT(a)   (CONST_UNIT(EXPR_CONST(a)))
#define EXPR_CONST_TYPE(a)   (CONST_TYPE(EXPR_CONST(a)))
#define EXPR_CONST_INT(a)    (CONST_INT(EXPR_CONST(a)))
#define EXPR_CONST_DOUBLE(a) (CONST_DOUBLE(EXPR_CONST(a)))
#define EXPR_CONST_STRING(a) (CONST_STRING(EXPR_CONST(a)))
#define EXPR_SYMINFO(a)      ((a)->u.thesyminfo)
#define EXPR_SYM(a)          (&((a)->u.thesyminfo->thesym))
#define EXPR_LASTSTRUCT(a)   (&((a)->u.thesyminfo->last_struct))
#define EXPR_BASESYM(a)      (&((a)->u.thesyminfo->basesym))
#define EXPR_TYPEDATA(a)     ((a)->u.thesyminfo->thedata)
#define EXPR_STRUCTOFFSET(a) ((a)->u.thesyminfo->structs_offset)
#define EXPR_OFFSET(a)       ((a)->u.thesyminfo->offset_from_base)
#define EXPR_IDENT(a)        ((a)->u.theident)
#define EXPR_OP(a)           ((a)->u.theopr)
#define EXPR_OP_TYPE(a)      (EXPR_OP(a)->theoper)
#define EXPR_OP_LEFT(a)      (EXPR_OP(a)->left)
#define EXPR_OP_RIGHT(a)     (EXPR_OP(a)->right)
#define EXPR_NEXT(a)         ((a)->next)
#define EXPR_IS_CONST(a)     (EXPR_TYPE(a) == CONST_EXPR)
#define EXPR_IS_OP(a)        (EXPR_TYPE(a) == OP_EXPR)
#define EXPR_IS_IDENT(a)     (EXPR_TYPE(a) == IDENT_EXPR)
#define EXPR_IS_SYMINFO(a)   (EXPR_TYPE(a) == SYMINFO_EXPR)
#define EXPR_IS_RETURN(a)    (EXPR_IS_OP(a) && EXPR_OP_TYPE(a) ==  RETURN)
#define EXPR_IS_ADDROF(a)    (EXPR_IS_OP(a) && EXPR_OP_TYPE(a) ==  ADDROF)
#define EXPR_IS_ASSIGN(a)    (EXPR_IS_OP(a) && EXPR_OP_TYPE(a) ==  EQUAL)
#define EXPR_IS_PLUS(a)      (EXPR_IS_OP(a) && EXPR_OP_TYPE(a) ==  PLUS)
#define EXPR_IS_MULT(a)      (EXPR_IS_OP(a) && EXPR_OP_TYPE(a) ==  MULT)
#define EXPR_IS_INT(a)       (EXPR_IS_CONST(a) && EXPR_CONST_TYPE(a)==INT_CONST)
#define EXPR_INT(a)          (CONST_INT (EXPR_CONST (a)))

#endif
